# -*- coding: utf-8 -*-

import base64
import os

from django.conf import settings
from django.core.files.storage import FileSystemStorage
from django.db import models
from django.utils import timezone
from django.utils.text import slugify
from django.utils.translation import gettext_lazy as _
from thx.core.models import CleanModel, DateModel, OrderedModel, StatusModel, UserModel

fs = FileSystemStorage(location=settings.MEDIA_PRIVATE_ROOT, base_url=settings.MEDIA_PRIVATE_URL)


def date_as_path() -> str:
    return f"{timezone.now():%Y/%m/%d/}"


def upload_attachment(instance, filename):
    instance.remove_attachments()
    return instance.get_filename_path(filename)


class Comment(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    title = models.CharField(
        max_length=255,
        blank=True,
        null=True,
        verbose_name=_('title'),
    )
    message = models.TextField(
        verbose_name=_('message'),
    )
    type = models.CharField(
        max_length=255,
        null=True,
        verbose_name=_('type')
    )
    is_public = models.BooleanField(
        default=True,
        verbose_name=_('is public'),
    )

    class Meta:
        verbose_name = _('comment')
        verbose_name_plural = _('comments')
        ordering = ('ordering',)
        permissions = (
            ("list_comment", "Can list comment"),
            ("detail_comment", "Can detail comment"),
            ("disable_comment", "Can disable comment"),
        )

    def __str__(self):
        return self.title if self.title else f"Comment id: {self.pk}"

    def save(self, *args, **kwargs):
        self.type = self._meta.model_name
        super().save()


class Attachment(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    remove_old_attachments = False
    name = models.CharField(
        max_length=255,
        verbose_name=_('name'),
    )
    type = models.CharField(
        max_length=255,
        null=True,
        verbose_name=_('type')
    )
    attachment = models.FileField(
        upload_to=upload_attachment,
        storage=fs,
    )
    description = models.CharField(
        max_length=255,
        blank=True, null=True,
        verbose_name=_('description'),
    )

    class Meta:
        verbose_name = _('attachment')
        verbose_name_plural = _('attachments')
        ordering = ('ordering', 'name',)
        permissions = (
            ("list_attachment", "Can list attachment"),
            ("detail_attachment", "Can detail attachment"),
            ("disable_attachment", "Can disable attachment"),
            ("download_attachment", "Can download attachment"),
        )

    def __str__(self):
        return self.name

    def save(self, *args, **kwargs):
        self.type = self._meta.model_name
        super().save()

    @property
    def attachment_base64(self):
        base64_file = None
        try:
            ext = self.attachment.name.split(".")[-1]
            # TODO Check this!
            if ext in ['jpg', 'jpeg', 'png']:
                with open(self.attachment.file.name, "rb") as image_file:
                    base64_file = base64.b64encode(image_file.read())
        except Exception:
            # TODO catch exceptions and log
            pass
        return base64_file

    def get_filename_path(self, filename):
        date_path = date_as_path()
        name, ext = os.path.splitext(filename)
        name = slugify(self.name)
        path = os.path.join(self.type, date_path, f"{name}{ext}")
        return path

    def remove_attachments(self):
        if self.remove_old_attachments:
            try:
                old_attachment = Attachment.objects.get(pk=self.pk)
                old_attachment.attachment.delete()
            except Exception:
                pass
